"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.NodeBackend = void 0;

var tslib_1 = _interopRequireWildcard(require("tslib"));

var _core = require("@sentry/core");

var _types = require("@sentry/types");

var _utils = require("@sentry/utils");

var _parsers = require("./parsers");

var _transports = require("./transports");

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

/**
 * The Sentry Node SDK Backend.
 * @hidden
 */
var NodeBackend =
/** @class */
function (_super) {
  tslib_1.__extends(NodeBackend, _super);

  function NodeBackend() {
    return _super !== null && _super.apply(this, arguments) || this;
  }
  /**
   * @inheritDoc
   */


  NodeBackend.prototype._setupTransport = function () {
    if (!this._options.dsn) {
      // We return the noop transport here in case there is no Dsn.
      return _super.prototype._setupTransport.call(this);
    }

    var dsn = new _core.Dsn(this._options.dsn);

    var transportOptions = tslib_1.__assign({}, this._options.transportOptions, this._options.httpProxy && {
      httpProxy: this._options.httpProxy
    }, this._options.httpsProxy && {
      httpsProxy: this._options.httpsProxy
    }, this._options.caCerts && {
      caCerts: this._options.caCerts
    }, {
      dsn: this._options.dsn
    });

    if (this._options.transport) {
      return new this._options.transport(transportOptions);
    }

    if (dsn.protocol === 'http') {
      return new _transports.HTTPTransport(transportOptions);
    }

    return new _transports.HTTPSTransport(transportOptions);
  };
  /**
   * @inheritDoc
   */


  NodeBackend.prototype.eventFromException = function (exception, hint) {
    var _this = this;

    var ex = exception;
    var mechanism = {
      handled: true,
      type: 'generic'
    };

    if (!(0, _utils.isError)(exception)) {
      if ((0, _utils.isPlainObject)(exception)) {
        // This will allow us to group events based on top-level keys
        // which is much better than creating new group when any key/value change
        var keys = Object.keys(exception).sort();
        var message = "Non-Error exception captured with keys: " + (0, _utils.keysToEventMessage)(keys);
        (0, _core.getCurrentHub)().configureScope(function (scope) {
          scope.setExtra('__serialized__', (0, _utils.normalizeToSize)(exception));
        });
        ex = hint && hint.syntheticException || new Error(message);
        ex.message = message;
      } else {
        // This handles when someone does: `throw "something awesome";`
        // We use synthesized Error here so we can extract a (rough) stack trace.
        ex = hint && hint.syntheticException || new Error(exception);
      }

      mechanism.synthetic = true;
    }

    return new _utils.SyncPromise(function (resolve, reject) {
      return (0, _parsers.parseError)(ex, _this._options).then(function (event) {
        (0, _utils.addExceptionTypeValue)(event, undefined, undefined, mechanism);
        resolve(tslib_1.__assign({}, event, {
          event_id: hint && hint.event_id
        }));
      }).catch(reject);
    });
  };
  /**
   * @inheritDoc
   */


  NodeBackend.prototype.eventFromMessage = function (message, level, hint) {
    var _this = this;

    if (level === void 0) {
      level = _types.Severity.Info;
    }

    var event = {
      event_id: hint && hint.event_id,
      level: level,
      message: message
    };
    return new _utils.SyncPromise(function (resolve) {
      if (_this._options.attachStacktrace && hint && hint.syntheticException) {
        var stack = hint.syntheticException ? (0, _parsers.extractStackFromError)(hint.syntheticException) : [];
        (0, _parsers.parseStack)(stack, _this._options).then(function (frames) {
          event.stacktrace = {
            frames: (0, _parsers.prepareFramesForEvent)(frames)
          };
          resolve(event);
        });
      } else {
        resolve(event);
      }
    });
  };

  return NodeBackend;
}(_core.BaseBackend);

exports.NodeBackend = NodeBackend;