"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Console = void 0;

var tslib_1 = _interopRequireWildcard(require("tslib"));

var _core = require("@sentry/core");

var _types = require("@sentry/types");

var _utils = require("@sentry/utils");

var util = _interopRequireWildcard(require("util"));

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

/** Console module integration */
var Console =
/** @class */
function () {
  function Console() {
    /**
     * @inheritDoc
     */
    this.name = Console.id;
  }
  /**
   * @inheritDoc
   */


  Console.prototype.setupOnce = function () {
    var nativeModule = require('module');

    (0, _utils.fill)(nativeModule, '_load', loadWrapper(nativeModule)); // special case: since console is built-in and app-level code won't require() it, do that here

    require('console');
  };
  /**
   * @inheritDoc
   */


  Console.id = 'Console';
  return Console;
}();

exports.Console = Console;

/**
 * Wrapper function for internal _load calls within `require`
 */
function loadWrapper(nativeModule) {
  // We need to use some functional-style currying to pass values around
  // as we cannot rely on `bind`, because this has to preserve correct
  // context for native calls
  return function (originalLoad) {
    return function (moduleId) {
      var originalModule = originalLoad.apply(nativeModule, arguments);

      if (moduleId !== 'console' || originalModule.__sentry__) {
        return originalModule;
      }

      ['debug', 'info', 'warn', 'error', 'log'].forEach(consoleWrapper(originalModule));
      originalModule.__sentry__ = true;
      return originalModule;
    };
  };
}
/**
 * Wrapper function that'll be used for every console level
 */


function consoleWrapper(originalModule) {
  return function (level) {
    if (!(level in originalModule)) {
      return;
    }

    (0, _utils.fill)(originalModule, level, function (originalConsoleLevel) {
      var sentryLevel;

      switch (level) {
        case 'debug':
          sentryLevel = _types.Severity.Debug;
          break;

        case 'error':
          sentryLevel = _types.Severity.Error;
          break;

        case 'info':
          sentryLevel = _types.Severity.Info;
          break;

        case 'warn':
          sentryLevel = _types.Severity.Warning;
          break;

        default:
          sentryLevel = _types.Severity.Log;
      }

      return function () {
        if ((0, _core.getCurrentHub)().getIntegration(Console)) {
          (0, _core.getCurrentHub)().addBreadcrumb({
            category: 'console',
            level: sentryLevel,
            message: util.format.apply(undefined, arguments)
          }, {
            input: tslib_1.__spread(arguments),
            level: level
          });
        }

        originalConsoleLevel.apply(originalModule, arguments);
      };
    });
  };
}