"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.LinkedErrors = void 0;

var tslib_1 = _interopRequireWildcard(require("tslib"));

var _core = require("@sentry/core");

var _utils = require("@sentry/utils");

var _parsers = require("../parsers");

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

var DEFAULT_KEY = 'cause';
var DEFAULT_LIMIT = 5;
/** Adds SDK info to an event. */

var LinkedErrors =
/** @class */
function () {
  /**
   * @inheritDoc
   */
  function LinkedErrors(options) {
    if (options === void 0) {
      options = {};
    }
    /**
     * @inheritDoc
     */


    this.name = LinkedErrors.id;
    this._key = options.key || DEFAULT_KEY;
    this._limit = options.limit || DEFAULT_LIMIT;
  }
  /**
   * @inheritDoc
   */


  LinkedErrors.prototype.setupOnce = function () {
    (0, _core.addGlobalEventProcessor)(function (event, hint) {
      var self = (0, _core.getCurrentHub)().getIntegration(LinkedErrors);

      if (self) {
        return self.handler(event, hint);
      }

      return event;
    });
  };
  /**
   * @inheritDoc
   */


  LinkedErrors.prototype.handler = function (event, hint) {
    var _this = this;

    if (!event.exception || !event.exception.values || !hint || !(hint.originalException instanceof Error)) {
      return _utils.SyncPromise.resolve(event);
    }

    return new _utils.SyncPromise(function (resolve) {
      _this.walkErrorTree(hint.originalException, _this._key).then(function (linkedErrors) {
        if (event && event.exception && event.exception.values) {
          event.exception.values = tslib_1.__spread(linkedErrors, event.exception.values);
        }

        resolve(event);
      });
    });
  };
  /**
   * @inheritDoc
   */


  LinkedErrors.prototype.walkErrorTree = function (error, key, stack) {
    var _this = this;

    if (stack === void 0) {
      stack = [];
    }

    if (!(error[key] instanceof Error) || stack.length + 1 >= this._limit) {
      return _utils.SyncPromise.resolve(stack);
    }

    return new _utils.SyncPromise(function (resolve) {
      (0, _parsers.getExceptionFromError)(error[key]).then(function (exception) {
        _this.walkErrorTree(error[key], key, tslib_1.__spread([exception], stack)).then(resolve);
      });
    });
  };
  /**
   * @inheritDoc
   */


  LinkedErrors.id = 'LinkedErrors';
  return LinkedErrors;
}();

exports.LinkedErrors = LinkedErrors;