"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.OnUnhandledRejection = void 0;

var _core = require("@sentry/core");

/** Global Promise Rejection handler */
var OnUnhandledRejection =
/** @class */
function () {
  function OnUnhandledRejection() {
    /**
     * @inheritDoc
     */
    this.name = OnUnhandledRejection.id;
  }
  /**
   * @inheritDoc
   */


  OnUnhandledRejection.prototype.setupOnce = function () {
    global.process.on('unhandledRejection', this.sendUnhandledPromise.bind(this));
  };
  /**
   * Send an exception with reason
   * @param reason string
   * @param promise promise
   */


  OnUnhandledRejection.prototype.sendUnhandledPromise = function (reason, promise) {
    var hub = (0, _core.getCurrentHub)();

    if (!hub.getIntegration(OnUnhandledRejection)) {
      return;
    }

    var context = promise.domain && promise.domain.sentryContext || {};
    hub.withScope(function (scope) {
      scope.setExtra('unhandledPromiseRejection', true); // Preserve backwards compatibility with raven-node for now

      if (context.user) {
        scope.setUser(context.user);
      }

      if (context.tags) {
        scope.setTags(context.tags);
      }

      if (context.extra) {
        scope.setExtras(context.extra);
      }

      hub.captureException(reason, {
        originalException: promise
      });
    });
  };
  /**
   * @inheritDoc
   */


  OnUnhandledRejection.id = 'OnUnhandledRejection';
  return OnUnhandledRejection;
}();

exports.OnUnhandledRejection = OnUnhandledRejection;