"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.BaseTransport = void 0;

var tslib_1 = _interopRequireWildcard(require("tslib"));

var _core = require("@sentry/core");

var _types = require("@sentry/types");

var _utils = require("@sentry/utils");

var fs = _interopRequireWildcard(require("fs"));

var _version = require("../version");

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

/** Base Transport class implementation */
var BaseTransport =
/** @class */
function () {
  /** Create instance and set this.dsn */
  function BaseTransport(options) {
    this.options = options;
    /** A simple buffer holding all requests. */

    this._buffer = new _utils.PromiseBuffer(30);
    this._api = new _core.API(options.dsn);
  }
  /** Returns a build request option object used by request */


  BaseTransport.prototype._getRequestOptions = function () {
    var headers = tslib_1.__assign({}, this._api.getRequestHeaders(_version.SDK_NAME, _version.SDK_VERSION), this.options.headers);

    var dsn = this._api.getDsn();

    var options = {
      agent: this.client,
      headers: headers,
      hostname: dsn.host,
      method: 'POST',
      path: this._api.getStoreEndpointPath(),
      port: dsn.port,
      protocol: dsn.protocol + ":"
    };

    if (this.options.caCerts) {
      options.ca = fs.readFileSync(this.options.caCerts);
    }

    return options;
  };
  /** JSDoc */


  BaseTransport.prototype._sendWithModule = function (httpModule, event) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
      var _this = this;

      return tslib_1.__generator(this, function (_a) {
        if (!this._buffer.isReady()) {
          return [2
          /*return*/
          , Promise.reject(new _utils.SentryError('Not adding Promise due to buffer limit reached.'))];
        }

        return [2
        /*return*/
        , this._buffer.add(new Promise(function (resolve, reject) {
          var req = httpModule.request(_this._getRequestOptions(), function (res) {
            res.setEncoding('utf8');

            if (res.statusCode && res.statusCode >= 200 && res.statusCode < 300) {
              resolve({
                status: _types.Status.fromHttpCode(res.statusCode)
              });
            } else {
              if (res.headers && res.headers['x-sentry-error']) {
                var reason = res.headers['x-sentry-error'];
                reject(new _utils.SentryError("HTTP Error (" + res.statusCode + "): " + reason));
              } else {
                reject(new _utils.SentryError("HTTP Error (" + res.statusCode + ")"));
              }
            } // force the socket to drain


            res.on('data', function () {// Drain
            });
            res.on('end', function () {// Drain
            });
          });
          req.on('error', reject);
          req.end(JSON.stringify(event));
        }))];
      });
    });
  };
  /**
   * @inheritDoc
   */


  BaseTransport.prototype.sendEvent = function (_) {
    throw new _utils.SentryError('Transport Class has to implement `sendEvent` method.');
  };
  /**
   * @inheritDoc
   */


  BaseTransport.prototype.close = function (timeout) {
    return this._buffer.drain(timeout);
  };

  return BaseTransport;
}();

exports.BaseTransport = BaseTransport;